{-# LANGUAGE PatternSynonyms, ScopedTypeVariables, ViewPatterns #-}


{- |
Copyright  : Will Thompson, Iñaki García Etxebarria and Jonas Platte
License    : LGPL-2.1
Maintainer : Iñaki García Etxebarria (inaki@blueleaf.cc)
-}

#define ENABLE_OVERLOADING (MIN_VERSION_haskell_gi_overloading(1,0,0) \
       && !defined(__HADDOCK_VERSION__))

module GI.Soup.Constants
    ( 
    pattern VERSION_MIN_REQUIRED            ,
    pattern SOCKET_USE_THREAD_CONTEXT       ,
    pattern SOCKET_TRUSTED_CERTIFICATE      ,
    pattern SOCKET_TLS_ERRORS               ,
    pattern SOCKET_TLS_CERTIFICATE          ,
    pattern SOCKET_TIMEOUT                  ,
    pattern SOCKET_SSL_STRICT               ,
    pattern SOCKET_SSL_FALLBACK             ,
    pattern SOCKET_SSL_CREDENTIALS          ,
    pattern SOCKET_REMOTE_ADDRESS           ,
    pattern SOCKET_LOCAL_ADDRESS            ,
    pattern SOCKET_IS_SERVER                ,
    pattern SOCKET_FLAG_NONBLOCKING         ,
    pattern SOCKET_ASYNC_CONTEXT            ,
    pattern SESSION_USE_THREAD_CONTEXT      ,
    pattern SESSION_USE_NTLM                ,
    pattern SESSION_USER_AGENT              ,
    pattern SESSION_TLS_INTERACTION         ,
    pattern SESSION_TLS_DATABASE            ,
    pattern SESSION_TIMEOUT                 ,
    pattern SESSION_SSL_USE_SYSTEM_CA_FILE  ,
    pattern SESSION_SSL_STRICT              ,
    pattern SESSION_SSL_CA_FILE             ,
    pattern SESSION_PROXY_URI               ,
    pattern SESSION_PROXY_RESOLVER          ,
    pattern SESSION_MAX_CONNS_PER_HOST      ,
    pattern SESSION_MAX_CONNS               ,
    pattern SESSION_LOCAL_ADDRESS           ,
    pattern SESSION_IDLE_TIMEOUT            ,
    pattern SESSION_HTTP_ALIASES            ,
    pattern SESSION_HTTPS_ALIASES           ,
    pattern SESSION_ASYNC_CONTEXT           ,
    pattern SESSION_ACCEPT_LANGUAGE_AUTO    ,
    pattern SESSION_ACCEPT_LANGUAGE         ,
    pattern SERVER_TLS_CERTIFICATE          ,
    pattern SERVER_SSL_KEY_FILE             ,
    pattern SERVER_SSL_CERT_FILE            ,
    pattern SERVER_SERVER_HEADER            ,
    pattern SERVER_RAW_PATHS                ,
    pattern SERVER_PORT                     ,
    pattern SERVER_INTERFACE                ,
    pattern SERVER_HTTP_ALIASES             ,
    pattern SERVER_HTTPS_ALIASES            ,
    pattern SERVER_ASYNC_CONTEXT            ,
    pattern REQUEST_URI                     ,
    pattern REQUEST_SESSION                 ,
    pattern MINOR_VERSION                   ,
    pattern MICRO_VERSION                   ,
    pattern MESSAGE_URI                     ,
    pattern MESSAGE_TLS_ERRORS              ,
    pattern MESSAGE_TLS_CERTIFICATE         ,
    pattern MESSAGE_STATUS_CODE             ,
    pattern MESSAGE_SERVER_SIDE             ,
    pattern MESSAGE_RESPONSE_HEADERS        ,
    pattern MESSAGE_RESPONSE_BODY_DATA      ,
    pattern MESSAGE_RESPONSE_BODY           ,
    pattern MESSAGE_REQUEST_HEADERS         ,
    pattern MESSAGE_REQUEST_BODY_DATA       ,
    pattern MESSAGE_REQUEST_BODY            ,
    pattern MESSAGE_REASON_PHRASE           ,
    pattern MESSAGE_PRIORITY                ,
    pattern MESSAGE_METHOD                  ,
    pattern MESSAGE_HTTP_VERSION            ,
    pattern MESSAGE_FLAGS                   ,
    pattern MESSAGE_FIRST_PARTY             ,
    pattern MAJOR_VERSION                   ,
    pattern LOGGER_MAX_BODY_SIZE            ,
    pattern LOGGER_LEVEL                    ,
    pattern FORM_MIME_TYPE_URLENCODED       ,
    pattern FORM_MIME_TYPE_MULTIPART        ,
    pattern COOKIE_MAX_AGE_ONE_YEAR         ,
    pattern COOKIE_MAX_AGE_ONE_WEEK         ,
    pattern COOKIE_MAX_AGE_ONE_HOUR         ,
    pattern COOKIE_MAX_AGE_ONE_DAY          ,
    pattern COOKIE_JAR_TEXT_FILENAME        ,
    pattern COOKIE_JAR_READ_ONLY            ,
    pattern COOKIE_JAR_DB_FILENAME          ,
    pattern COOKIE_JAR_ACCEPT_POLICY        ,
    pattern CHAR_URI_SUB_DELIMS             ,
    pattern CHAR_URI_PERCENT_ENCODED        ,
    pattern CHAR_URI_GEN_DELIMS             ,
    pattern CHAR_HTTP_SEPARATOR             ,
    pattern CHAR_HTTP_CTL                   ,
    pattern AUTH_SCHEME_NAME                ,
    pattern AUTH_REALM                      ,
    pattern AUTH_IS_FOR_PROXY               ,
    pattern AUTH_IS_AUTHENTICATED           ,
    pattern AUTH_HOST                       ,
    pattern AUTH_DOMAIN_REMOVE_PATH         ,
    pattern AUTH_DOMAIN_REALM               ,
    pattern AUTH_DOMAIN_PROXY               ,
    pattern AUTH_DOMAIN_GENERIC_AUTH_DATA   ,
    pattern AUTH_DOMAIN_GENERIC_AUTH_CALLBACK,
    pattern AUTH_DOMAIN_FILTER_DATA         ,
    pattern AUTH_DOMAIN_FILTER              ,
    pattern AUTH_DOMAIN_DIGEST_AUTH_DATA    ,
    pattern AUTH_DOMAIN_DIGEST_AUTH_CALLBACK,
    pattern AUTH_DOMAIN_BASIC_AUTH_DATA     ,
    pattern AUTH_DOMAIN_BASIC_AUTH_CALLBACK ,
    pattern AUTH_DOMAIN_ADD_PATH            ,
    pattern ADDRESS_SOCKADDR                ,
    pattern ADDRESS_PROTOCOL                ,
    pattern ADDRESS_PORT                    ,
    pattern ADDRESS_PHYSICAL                ,
    pattern ADDRESS_NAME                    ,
    pattern ADDRESS_FAMILY                  ,
    pattern ADDRESS_ANY_PORT                ,

    ) where

import Data.GI.Base.ShortPrelude
import qualified Data.GI.Base.ShortPrelude as SP
import qualified Data.GI.Base.Overloading as O
import qualified Prelude as P

import qualified Data.GI.Base.Attributes as GI.Attributes
import qualified Data.GI.Base.ManagedPtr as B.ManagedPtr
import qualified Data.GI.Base.GClosure as B.GClosure
import qualified Data.GI.Base.GError as B.GError
import qualified Data.GI.Base.GVariant as B.GVariant
import qualified Data.GI.Base.GValue as B.GValue
import qualified Data.GI.Base.GParamSpec as B.GParamSpec
import qualified Data.GI.Base.CallStack as B.CallStack
import qualified Data.GI.Base.Properties as B.Properties
import qualified Data.Text as T
import qualified Data.ByteString.Char8 as B
import qualified Data.Map as Map
import qualified Foreign.Ptr as FP
import qualified GHC.OverloadedLabels as OL


{- |
A macro that should be defined by the user prior to including
libsoup.h. The definition should be one of the predefined libsoup
version macros: @/SOUP_VERSION_2_24/@, @/SOUP_VERSION_2_26/@, ...

This macro defines the earliest version of libsoup that the package
is required to be able to compile against.

If the compiler is configured to warn about the use of deprecated
functions, then using functions that were deprecated in version
'GI.Soup.Constants.VERSION_MIN_REQUIRED' or earlier will cause warnings (but
using functions deprecated in later releases will not).

/Since: 2.42/
-}
pattern VERSION_MIN_REQUIRED = 2 :: Int32

{- |
Alias for the 'GI.Soup.Objects.Socket.Socket':@/use-thread-context/@ property. (Use
'GI.GLib.Functions.mainContextGetThreadDefault')

/Since: 2.38/
-}
pattern SOCKET_USE_THREAD_CONTEXT = "use-thread-context" :: T.Text

{- |
Alias for the 'GI.Soup.Objects.Socket.Socket':@/trusted-certificate/@
property.
-}
pattern SOCKET_TRUSTED_CERTIFICATE = "trusted-certificate" :: T.Text

{- |
Alias for the 'GI.Soup.Objects.Socket.Socket':@/tls-errors/@
property. Note that this property\'s value is only useful
if the socket is for a TLS connection, and only reliable
after some data has been transferred to or from it.

/Since: 2.34/
-}
pattern SOCKET_TLS_ERRORS = "tls-errors" :: T.Text

{- |
Alias for the 'GI.Soup.Objects.Socket.Socket':@/tls-certificate/@
property. Note that this property\'s value is only useful
if the socket is for a TLS connection, and only reliable
after some data has been transferred to or from it.

/Since: 2.34/
-}
pattern SOCKET_TLS_CERTIFICATE = "tls-certificate" :: T.Text

{- |
Alias for the 'GI.Soup.Objects.Socket.Socket':@/timeout/@ property. (The timeout
in seconds for blocking socket I\/O operations.)
-}
pattern SOCKET_TIMEOUT = "timeout" :: T.Text

{- |
Alias for the 'GI.Soup.Objects.Socket.Socket':@/ssl-strict/@ property.
-}
pattern SOCKET_SSL_STRICT = "ssl-strict" :: T.Text

{- |
Alias for the 'GI.Soup.Objects.Socket.Socket':@/ssl-fallback/@ property.
-}
pattern SOCKET_SSL_FALLBACK = "ssl-fallback" :: T.Text

{- |
Alias for the 'GI.Soup.Objects.Socket.Socket':@/ssl-creds/@ property.
(SSL credential information.)
-}
pattern SOCKET_SSL_CREDENTIALS = "ssl-creds" :: T.Text

{- |
Alias for the 'GI.Soup.Objects.Socket.Socket':@/remote-address/@ property. (Address
of remote end of socket.)
-}
pattern SOCKET_REMOTE_ADDRESS = "remote-address" :: T.Text

{- |
Alias for the 'GI.Soup.Objects.Socket.Socket':@/local-address/@ property. (Address
of local end of socket.)
-}
pattern SOCKET_LOCAL_ADDRESS = "local-address" :: T.Text

{- |
Alias for the 'GI.Soup.Objects.Socket.Socket':@/is-server/@ property, qv.
-}
pattern SOCKET_IS_SERVER = "is-server" :: T.Text

{- |
Alias for the 'GI.Soup.Objects.Socket.Socket':@/non-blocking/@ property. (Whether
or not the socket uses non-blocking I\/O.)
-}
pattern SOCKET_FLAG_NONBLOCKING = "non-blocking" :: T.Text

{- |
Alias for the 'GI.Soup.Objects.Socket.Socket':@/async-context/@ property. (The
socket\'s 'GI.GLib.Structs.MainContext.MainContext'.)
-}
pattern SOCKET_ASYNC_CONTEXT = "async-context" :: T.Text

{- |
Alias for the 'GI.Soup.Objects.Session.Session':@/use-thread-context/@ property, qv.

/Since: 2.38/
-}
pattern SESSION_USE_THREAD_CONTEXT = "use-thread-context" :: T.Text

{- |
Alias for the 'GI.Soup.Objects.Session.Session':@/use-ntlm/@ property, qv.
-}
pattern SESSION_USE_NTLM = "use-ntlm" :: T.Text

{- |
Alias for the 'GI.Soup.Objects.Session.Session':@/user-agent/@ property, qv.
-}
pattern SESSION_USER_AGENT = "user-agent" :: T.Text

{- |
Alias for the 'GI.Soup.Objects.Session.Session':@/tls-interaction/@ property, qv.

/Since: 2.48/
-}
pattern SESSION_TLS_INTERACTION = "tls-interaction" :: T.Text

{- |
Alias for the 'GI.Soup.Objects.Session.Session':@/tls-database/@ property, qv.

/Since: 2.38/
-}
pattern SESSION_TLS_DATABASE = "tls-database" :: T.Text

{- |
Alias for the 'GI.Soup.Objects.Session.Session':@/timeout/@ property, qv.
-}
pattern SESSION_TIMEOUT = "timeout" :: T.Text

{- |
Alias for the 'GI.Soup.Objects.Session.Session':@/ssl-use-system-ca-file/@ property,
qv.

/Since: 2.38/
-}
pattern SESSION_SSL_USE_SYSTEM_CA_FILE = "ssl-use-system-ca-file" :: T.Text

{- |
Alias for the 'GI.Soup.Objects.Session.Session':@/ssl-strict/@ property, qv.

/Since: 2.30/
-}
pattern SESSION_SSL_STRICT = "ssl-strict" :: T.Text

{- |
Alias for the 'GI.Soup.Objects.Session.Session':@/ssl-ca-file/@ property, qv.
-}
pattern SESSION_SSL_CA_FILE = "ssl-ca-file" :: T.Text

{- |
Alias for the 'GI.Soup.Objects.Session.Session':@/proxy-uri/@ property, qv.
-}
pattern SESSION_PROXY_URI = "proxy-uri" :: T.Text

{- |
Alias for the 'GI.Soup.Objects.Session.Session':@/proxy-resolver/@ property, qv.
-}
pattern SESSION_PROXY_RESOLVER = "proxy-resolver" :: T.Text

{- |
Alias for the 'GI.Soup.Objects.Session.Session':@/max-conns-per-host/@ property, qv.
-}
pattern SESSION_MAX_CONNS_PER_HOST = "max-conns-per-host" :: T.Text

{- |
Alias for the 'GI.Soup.Objects.Session.Session':@/max-conns/@ property, qv.
-}
pattern SESSION_MAX_CONNS = "max-conns" :: T.Text

{- |
Alias for the 'GI.Soup.Objects.Session.Session':@/local-address/@ property, qv.

/Since: 2.42/
-}
pattern SESSION_LOCAL_ADDRESS = "local-address" :: T.Text

{- |
Alias for the 'GI.Soup.Objects.Session.Session':@/idle-timeout/@ property, qv.

/Since: 2.24/
-}
pattern SESSION_IDLE_TIMEOUT = "idle-timeout" :: T.Text

{- |
Alias for the 'GI.Soup.Objects.Session.Session':@/http-aliases/@ property, qv.

/Since: 2.38/
-}
pattern SESSION_HTTP_ALIASES = "http-aliases" :: T.Text

{- |
Alias for the 'GI.Soup.Objects.Session.Session':@/https-aliases/@ property, qv.

/Since: 2.38/
-}
pattern SESSION_HTTPS_ALIASES = "https-aliases" :: T.Text

{- |
Alias for the 'GI.Soup.Objects.Session.Session':@/async-context/@ property, qv.
-}
pattern SESSION_ASYNC_CONTEXT = "async-context" :: T.Text

{- |
Alias for the 'GI.Soup.Objects.Session.Session':@/accept-language-auto/@ property, qv.

/Since: 2.30/
-}
pattern SESSION_ACCEPT_LANGUAGE_AUTO = "accept-language-auto" :: T.Text

{- |
Alias for the 'GI.Soup.Objects.Session.Session':@/accept-language/@ property, qv.

/Since: 2.30/
-}
pattern SESSION_ACCEPT_LANGUAGE = "accept-language" :: T.Text

{- |
Alias for the 'GI.Soup.Objects.Server.Server':@/tls-certificate/@ property, qv.

/Since: 2.38/
-}
pattern SERVER_TLS_CERTIFICATE = "tls-certificate" :: T.Text

{-# DEPRECATED SERVER_SSL_KEY_FILE ["use 'GI.Soup.Objects.Server.Server':@/tls-certificate/@ or","@/soup_server_set_ssl_certificate()/@."] #-}
{- |
Alias for the 'GI.Soup.Objects.Server.Server':@/ssl-key-file/@ property, qv.
-}
pattern SERVER_SSL_KEY_FILE = "ssl-key-file" :: T.Text

{-# DEPRECATED SERVER_SSL_CERT_FILE ["use 'GI.Soup.Objects.Server.Server':@/tls-certificate/@ or","@/soup_server_set_ssl_certificate()/@."] #-}
{- |
Alias for the 'GI.Soup.Objects.Server.Server':@/ssl-cert-file/@ property, qv.
-}
pattern SERVER_SSL_CERT_FILE = "ssl-cert-file" :: T.Text

{- |
Alias for the 'GI.Soup.Objects.Server.Server':@/server-header/@ property, qv.
-}
pattern SERVER_SERVER_HEADER = "server-header" :: T.Text

{- |
Alias for the 'GI.Soup.Objects.Server.Server':@/raw-paths/@ property. (If 'True',
percent-encoding in the Request-URI path will not be
automatically decoded.)
-}
pattern SERVER_RAW_PATHS = "raw-paths" :: T.Text

{-# DEPRECATED SERVER_PORT ["@/SoupServers/@ can listen on multiple interfaces","at once now. Use 'GI.Soup.Objects.Server.serverListen', etc, to listen on a","port, and 'GI.Soup.Objects.Server.serverGetUris' to see what ports are","being listened on."] #-}
{- |
Alias for the deprecated 'GI.Soup.Objects.Server.Server':@/port/@ property, qv.
-}
pattern SERVER_PORT = "port" :: T.Text

{-# DEPRECATED SERVER_INTERFACE ["@/SoupServers/@ can listen on multiple interfaces","at once now. Use 'GI.Soup.Objects.Server.serverListen', etc, to listen on an","interface, and 'GI.Soup.Objects.Server.serverGetUris' to see what addresses","are being listened on."] #-}
{- |
Alias for the 'GI.Soup.Objects.Server.Server':@/interface/@ property, qv.
-}
pattern SERVER_INTERFACE = "interface" :: T.Text

{- |
Alias for the 'GI.Soup.Objects.Server.Server':@/http-aliases/@ property, qv.

/Since: 2.44/
-}
pattern SERVER_HTTP_ALIASES = "http-aliases" :: T.Text

{- |
Alias for the 'GI.Soup.Objects.Server.Server':@/https-aliases/@ property, qv.

/Since: 2.44/
-}
pattern SERVER_HTTPS_ALIASES = "https-aliases" :: T.Text

{-# DEPRECATED SERVER_ASYNC_CONTEXT ["The new API uses the thread-default 'GI.GLib.Structs.MainContext.MainContext'","rather than having an explicitly-specified one."] #-}
{- |
Alias for the deprecated 'GI.Soup.Objects.Server.Server':@/async-context/@
property, qv.
-}
pattern SERVER_ASYNC_CONTEXT = "async-context" :: T.Text

{- |
Alias for the 'GI.Soup.Objects.Request.Request':@/uri/@ property, qv.

/Since: 2.42/
-}
pattern REQUEST_URI = "uri" :: T.Text

{- |
Alias for the 'GI.Soup.Objects.Request.Request':@/session/@ property, qv.

/Since: 2.42/
-}
pattern REQUEST_SESSION = "session" :: T.Text

{- |
Like 'GI.Soup.Functions.getMinorVersion', but from the headers used at
application compile time, rather than from the library linked
against at application run time.

/Since: 2.42/
-}
pattern MINOR_VERSION = 66 :: Int32

{- |
Like 'GI.Soup.Functions.getMicroVersion', but from the headers used at
application compile time, rather than from the library linked
against at application run time.

/Since: 2.42/
-}
pattern MICRO_VERSION = 2 :: Int32

{- |
Alias for the 'GI.Soup.Objects.Message.Message':@/uri/@ property. (The message\'s
'GI.Soup.Structs.URI.URI'.)
-}
pattern MESSAGE_URI = "uri" :: T.Text

{- |
Alias for the 'GI.Soup.Objects.Message.Message':@/tls-errors/@ property. (The
verification errors on 'GI.Soup.Objects.Message.Message':@/tls-certificate/@.)

/Since: 2.34/
-}
pattern MESSAGE_TLS_ERRORS = "tls-errors" :: T.Text

{- |
Alias for the 'GI.Soup.Objects.Message.Message':@/tls-certificate/@ property. (The
TLS certificate associated with the message, if any.)

/Since: 2.34/
-}
pattern MESSAGE_TLS_CERTIFICATE = "tls-certificate" :: T.Text

{- |
Alias for the 'GI.Soup.Objects.Message.Message':@/status-code/@ property. (The
message\'s HTTP response status code.)
-}
pattern MESSAGE_STATUS_CODE = "status-code" :: T.Text

{- |
Alias for the 'GI.Soup.Objects.Message.Message':@/server-side/@ property. ('True' if
the message was created by 'GI.Soup.Objects.Server.Server'.)
-}
pattern MESSAGE_SERVER_SIDE = "server-side" :: T.Text

{- |
Alias for the 'GI.Soup.Objects.Message.Message':@/response-headers/@ property. (The
message\'s HTTP response headers.)
-}
pattern MESSAGE_RESPONSE_HEADERS = "response-headers" :: T.Text

{- |
Alias for the 'GI.Soup.Objects.Message.Message':@/response-body-data/@ property. (The
message\'s HTTP response body, as a 'GI.GLib.Structs.Bytes.Bytes'.)

/Since: 2.46/
-}
pattern MESSAGE_RESPONSE_BODY_DATA = "response-body-data" :: T.Text

{- |
Alias for the 'GI.Soup.Objects.Message.Message':@/response-body/@ property. (The
message\'s HTTP response body.)
-}
pattern MESSAGE_RESPONSE_BODY = "response-body" :: T.Text

{- |
Alias for the 'GI.Soup.Objects.Message.Message':@/request-headers/@ property. (The
message\'s HTTP request headers.)
-}
pattern MESSAGE_REQUEST_HEADERS = "request-headers" :: T.Text

{- |
Alias for the 'GI.Soup.Objects.Message.Message':@/request-body-data/@ property. (The
message\'s HTTP request body, as a 'GI.GLib.Structs.Bytes.Bytes'.)

/Since: 2.46/
-}
pattern MESSAGE_REQUEST_BODY_DATA = "request-body-data" :: T.Text

{- |
Alias for the 'GI.Soup.Objects.Message.Message':@/request-body/@ property. (The
message\'s HTTP request body.)
-}
pattern MESSAGE_REQUEST_BODY = "request-body" :: T.Text

{- |
Alias for the 'GI.Soup.Objects.Message.Message':@/reason-phrase/@ property. (The
message\'s HTTP response reason phrase.)
-}
pattern MESSAGE_REASON_PHRASE = "reason-phrase" :: T.Text

{- |
Sets the priority of the 'GI.Soup.Objects.Message.Message'. See
'GI.Soup.Objects.Message.messageSetPriority' for further details.

/Since: 2.44/
-}
pattern MESSAGE_PRIORITY = "priority" :: T.Text

{- |
Alias for the 'GI.Soup.Objects.Message.Message':@/method/@ property. (The message\'s
HTTP method.)
-}
pattern MESSAGE_METHOD = "method" :: T.Text

{- |
Alias for the 'GI.Soup.Objects.Message.Message':@/http-version/@ property. (The
message\'s 'GI.Soup.Enums.HTTPVersion'.)
-}
pattern MESSAGE_HTTP_VERSION = "http-version" :: T.Text

{- |
Alias for the 'GI.Soup.Objects.Message.Message':@/flags/@ property. (The message\'s
'GI.Soup.Flags.MessageFlags'.)
-}
pattern MESSAGE_FLAGS = "flags" :: T.Text

{- |
Alias for the 'GI.Soup.Objects.Message.Message':@/first-party/@ property. (The
'GI.Soup.Structs.URI.URI' loaded in the application when the message was
queued.)

/Since: 2.30/
-}
pattern MESSAGE_FIRST_PARTY = "first-party" :: T.Text

{- |
Like 'GI.Soup.Functions.getMajorVersion', but from the headers used at
application compile time, rather than from the library linked
against at application run time.

/Since: 2.42/
-}
pattern MAJOR_VERSION = 2 :: Int32

{- |
Alias for the 'GI.Soup.Objects.Logger.Logger':@/max-body-size/@ property, qv.

/Since: 2.56/
-}
pattern LOGGER_MAX_BODY_SIZE = "max-body-size" :: T.Text

{- |
Alias for the 'GI.Soup.Objects.Logger.Logger':@/level/@ property, qv.

/Since: 2.56/
-}
pattern LOGGER_LEVEL = "level" :: T.Text

{- |
A macro containing the value
\<literal>\"application\/x-www-form-urlencoded\"\<\/literal>; the default
MIME type for POSTing HTML form data.

/Since: 2.26/
-}
pattern FORM_MIME_TYPE_URLENCODED = "application/x-www-form-urlencoded" :: T.Text

{- |
A macro containing the value
\<literal>\"multipart\/form-data\"\<\/literal>; the MIME type used for
posting form data that contains files to be uploaded.

/Since: 2.26/
-}
pattern FORM_MIME_TYPE_MULTIPART = "multipart/form-data" :: T.Text

{- |
A constant corresponding to 1 year, for use with 'GI.Soup.Structs.Cookie.cookieNew'
and 'GI.Soup.Structs.Cookie.cookieSetMaxAge'.

/Since: 2.24/
-}
pattern COOKIE_MAX_AGE_ONE_YEAR = 0 :: Int32

{- |
A constant corresponding to 1 week, for use with 'GI.Soup.Structs.Cookie.cookieNew'
and 'GI.Soup.Structs.Cookie.cookieSetMaxAge'.

/Since: 2.24/
-}
pattern COOKIE_MAX_AGE_ONE_WEEK = 0 :: Int32

{- |
A constant corresponding to 1 hour, for use with 'GI.Soup.Structs.Cookie.cookieNew'
and 'GI.Soup.Structs.Cookie.cookieSetMaxAge'.

/Since: 2.24/
-}
pattern COOKIE_MAX_AGE_ONE_HOUR = 3600 :: Int32

{- |
A constant corresponding to 1 day, for use with 'GI.Soup.Structs.Cookie.cookieNew'
and 'GI.Soup.Structs.Cookie.cookieSetMaxAge'.

/Since: 2.24/
-}
pattern COOKIE_MAX_AGE_ONE_DAY = 0 :: Int32

{- |
Alias for the 'GI.Soup.Objects.CookieJarText.CookieJarText':@/filename/@ property. (The
cookie-storage filename.)
-}
pattern COOKIE_JAR_TEXT_FILENAME = "filename" :: T.Text

{- |
Alias for the 'GI.Soup.Objects.CookieJar.CookieJar':@/read-only/@ property. (Whether
or not the cookie jar is read-only.)
-}
pattern COOKIE_JAR_READ_ONLY = "read-only" :: T.Text

{- |
Alias for the 'GI.Soup.Objects.CookieJarDB.CookieJarDB':@/filename/@ property. (The
cookie-storage filename.)
-}
pattern COOKIE_JAR_DB_FILENAME = "filename" :: T.Text

{- |
Alias for the 'GI.Soup.Objects.CookieJar.CookieJar':@/accept-policy/@ property.

/Since: 2.30/
-}
pattern COOKIE_JAR_ACCEPT_POLICY = "accept-policy" :: T.Text

{- |
/No description available in the introspection data./
-}
pattern CHAR_URI_SUB_DELIMS = 4 :: Int32

{- |
/No description available in the introspection data./
-}
pattern CHAR_URI_PERCENT_ENCODED = 1 :: Int32

{- |
/No description available in the introspection data./
-}
pattern CHAR_URI_GEN_DELIMS = 2 :: Int32

{- |
/No description available in the introspection data./
-}
pattern CHAR_HTTP_SEPARATOR = 8 :: Int32

{- |
/No description available in the introspection data./
-}
pattern CHAR_HTTP_CTL = 16 :: Int32

{- |
An alias for the 'GI.Soup.Objects.Auth.Auth':@/scheme-name/@ property. (The
authentication scheme name.)
-}
pattern AUTH_SCHEME_NAME = "scheme-name" :: T.Text

{- |
An alias for the 'GI.Soup.Objects.Auth.Auth':@/realm/@ property. (The
authentication realm.)
-}
pattern AUTH_REALM = "realm" :: T.Text

{- |
An alias for the 'GI.Soup.Objects.Auth.Auth':@/is-for-proxy/@ property. (Whether
or not the auth is for a proxy server.)
-}
pattern AUTH_IS_FOR_PROXY = "is-for-proxy" :: T.Text

{- |
An alias for the 'GI.Soup.Objects.Auth.Auth':@/is-authenticated/@ property.
(Whether or not the auth has been authenticated.)
-}
pattern AUTH_IS_AUTHENTICATED = "is-authenticated" :: T.Text

{- |
An alias for the 'GI.Soup.Objects.Auth.Auth':@/host/@ property. (The
host being authenticated to.)
-}
pattern AUTH_HOST = "host" :: T.Text

{- |
Alias for the 'GI.Soup.Objects.AuthDomain.AuthDomain':@/remove-path/@ property.
(Shortcut for calling 'GI.Soup.Objects.AuthDomain.authDomainRemovePath'.)
-}
pattern AUTH_DOMAIN_REMOVE_PATH = "remove-path" :: T.Text

{- |
Alias for the 'GI.Soup.Objects.AuthDomain.AuthDomain':@/realm/@ property. (The realm of
this auth domain.)
-}
pattern AUTH_DOMAIN_REALM = "realm" :: T.Text

{- |
Alias for the 'GI.Soup.Objects.AuthDomain.AuthDomain':@/proxy/@ property. (Whether or
not this is a proxy auth domain.)
-}
pattern AUTH_DOMAIN_PROXY = "proxy" :: T.Text

{- |
Alias for the 'GI.Soup.Objects.AuthDomain.AuthDomain':@/generic-auth-data/@ property.
(The data to pass to the 'GI.Soup.Callbacks.AuthDomainGenericAuthCallback'.)
-}
pattern AUTH_DOMAIN_GENERIC_AUTH_DATA = "generic-auth-data" :: T.Text

{- |
Alias for the 'GI.Soup.Objects.AuthDomain.AuthDomain':@/generic-auth-callback/@ property.
(The 'GI.Soup.Callbacks.AuthDomainGenericAuthCallback'.)
-}
pattern AUTH_DOMAIN_GENERIC_AUTH_CALLBACK = "generic-auth-callback" :: T.Text

{- |
Alias for the 'GI.Soup.Objects.AuthDomain.AuthDomain':@/filter-data/@ property. (Data
to pass to the 'GI.Soup.Callbacks.AuthDomainFilter'.)
-}
pattern AUTH_DOMAIN_FILTER_DATA = "filter-data" :: T.Text

{- |
Alias for the 'GI.Soup.Objects.AuthDomain.AuthDomain':@/filter/@ property. (The
'GI.Soup.Callbacks.AuthDomainFilter' for the domain.)
-}
pattern AUTH_DOMAIN_FILTER = "filter" :: T.Text

{- |
Alias for the 'GI.Soup.Objects.AuthDomainDigest.AuthDomainDigest':@/auth-callback/@ property.
(The 'GI.Soup.Callbacks.AuthDomainDigestAuthCallback'.)
-}
pattern AUTH_DOMAIN_DIGEST_AUTH_DATA = "auth-data" :: T.Text

{- |
Alias for the 'GI.Soup.Objects.AuthDomainDigest.AuthDomainDigest':@/auth-callback/@ property.
(The 'GI.Soup.Callbacks.AuthDomainDigestAuthCallback'.)
-}
pattern AUTH_DOMAIN_DIGEST_AUTH_CALLBACK = "auth-callback" :: T.Text

{- |
Alias for the 'GI.Soup.Objects.AuthDomainBasic.AuthDomainBasic':@/auth-data/@ property.
(The data to pass to the 'GI.Soup.Callbacks.AuthDomainBasicAuthCallback'.)
-}
pattern AUTH_DOMAIN_BASIC_AUTH_DATA = "auth-data" :: T.Text

{- |
Alias for the 'GI.Soup.Objects.AuthDomainBasic.AuthDomainBasic':@/auth-callback/@ property.
(The 'GI.Soup.Callbacks.AuthDomainBasicAuthCallback'.)
-}
pattern AUTH_DOMAIN_BASIC_AUTH_CALLBACK = "auth-callback" :: T.Text

{- |
Alias for the 'GI.Soup.Objects.AuthDomain.AuthDomain':@/add-path/@ property. (Shortcut
for calling 'GI.Soup.Objects.AuthDomain.authDomainAddPath'.)
-}
pattern AUTH_DOMAIN_ADD_PATH = "add-path" :: T.Text

{- |
An alias for the 'GI.Soup.Objects.Address.Address':@/sockaddr/@ property. (A pointer
to the struct sockaddr for this address.)
-}
pattern ADDRESS_SOCKADDR = "sockaddr" :: T.Text

{- |
Alias for the 'GI.Soup.Objects.Address.Address':@/protocol/@ property. (The URI scheme
used with this address.)
-}
pattern ADDRESS_PROTOCOL = "protocol" :: T.Text

{- |
An alias for the 'GI.Soup.Objects.Address.Address':@/port/@ property. (The port for
this address.)
-}
pattern ADDRESS_PORT = "port" :: T.Text

{- |
An alias for the 'GI.Soup.Objects.Address.Address':@/physical/@ property. (The
stringified IP address for this address.)
-}
pattern ADDRESS_PHYSICAL = "physical" :: T.Text

{- |
Alias for the 'GI.Soup.Objects.Address.Address':@/name/@ property. (The hostname for
this address.)
-}
pattern ADDRESS_NAME = "name" :: T.Text

{- |
Alias for the 'GI.Soup.Objects.Address.Address':@/family/@ property. (The
'GI.Soup.Enums.AddressFamily' for this address.)
-}
pattern ADDRESS_FAMILY = "family" :: T.Text

{- |
This can be passed to any 'GI.Soup.Objects.Address.Address' method that expects a port,
to indicate that you don\'t care what port is used.
-}
pattern ADDRESS_ANY_PORT = 0 :: Int32


